;________________________________________________________________________
;
;                                R A M . A S M
;________________________________________________________________________
;
; Ramtron FM25160 FRAM serial memory routines for PIC
; by Byon Garrabrant  N6BG  byon@mail.com  Jul 99
;________________________________________________________________________
;
;   This software is  available only for non-commercial  amateur radio 
;   or educational  applications.   ALL  other  uses  are  prohibited.  
;   This  software may  be  modified  only  if the  resulting code  be 
;   made available  publicly  and the  original  author given  credit.  
;   Contact the author or TAPR for information on commercial licenses. 
;   (c) Copyright 1999,  Byon Garrabrant,  N6BG.  All rights reserved. 
;________________________________________________________________________

	cblock				; variables
		RAMAddressHigh
		RAMAddressLow
		RAMSave
		RAMData
		RAMCount
	endc
;________________________________________________________________________

#define	RAM_CHIP_SELECT		PORTA, 0
#define	RAM_DATA_IN		PORTA, 1
#define	RAM_DATA_OUT		PORTA, 1
#define	RAM_CLOCK 		PORTA, 2

#define	RAM_OP_WREN		B'00000110'
#define	RAM_OP_WRSR		B'00000001'
#define	RAM_OP_READ		B'00000011'
#define	RAM_OP_WRITE  		B'00000010'
#define	RAM_STATUS_REG		B'10000000'
;________________________________________________________________________

RAMInit
	bsf	STATUS, RP0		; Select Bank 1
	bcf	RAM_CLOCK		; make output
	bcf	RAM_CHIP_SELECT		; make output
	bcf	STATUS, RP0		; Select Bank 0
	clrf	RAMAddressHigh
	clrf	RAMAddressLow
	call	RAMWriteEnable
	bcf	RAM_CHIP_SELECT
	movlw	RAM_OP_WRSR
	call	RAMWrite
	movlw	RAM_STATUS_REG
	call	RAMWrite
	bsf	RAM_CHIP_SELECT
	return
;________________________________________________________________________

RAMWrite
	bsf	STATUS, RP0		; Select Bank 1
	bcf	RAM_DATA_OUT		; make output
	bcf	STATUS, RP0		; Select Bank 0
	movwf	RAMData
	movlw	D'8'
	movwf	RAMCount
RAMSendLoop
	rlf	RAMData, F
	bcf	RAM_DATA_OUT
	skpnc
	 bsf	RAM_DATA_OUT
	bsf	RAM_CLOCK
	bcf	RAM_CLOCK
	decfsz	RAMCount, F
	 goto	RAMSendLoop
	return
;________________________________________________________________________

RAMRead
	bsf	STATUS, RP0		; Select Bank 1
	bsf	RAM_DATA_IN		; make input
	bcf	STATUS, RP0		; Select Bank 0
	movlw	D'8'
	movwf	RAMCount
RAMGetLoop
	bsf	RAM_CLOCK
	clrc
	btfsc	RAM_DATA_IN
	 setc
	rlf	RAMData, F
	bcf	RAM_CLOCK
	decfsz	RAMCount, F
	 goto	RAMGetLoop
	movfw	RAMData
	return
;________________________________________________________________________

RAMWriteEnable
	bcf	RAM_CHIP_SELECT
	movlw	RAM_OP_WREN
	call	RAMWrite
RAMStop
	bsf	RAM_CHIP_SELECT
	return
;________________________________________________________________________

RAMWriteStart
	call	RAMWriteEnable
	bcf	RAM_CHIP_SELECT
	swapf	RAMAddressHigh, W
	movwf	RAMData
	rrf	RAMData, W
	andlw	B'00111000'
	iorlw	RAM_OP_WRITE
	call	RAMWrite
	movfw	RAMAddressLow
	call	RAMWrite
	return

RAMWriteByte
	movwf	RAMSave
	call	RAMWriteStart
	movfw	RAMSave
	call	RAMWrite
	goto	RAMStop

;________________________________________________________________________

RAMReadStart
	bcf	RAM_CHIP_SELECT
	swapf	RAMAddressHigh, W
	movwf	RAMData
	rrf	RAMData, W
	andlw	B'00111000'
	iorlw	RAM_OP_READ
	call	RAMWrite
	movfw	RAMAddressLow
	call	RAMWrite
	return

RAMReadByte
	call	RAMReadStart
	call	RAMRead
	goto	RAMStop

;________________________________________________________________________
